      program calcT0

c     -- getting an updated period for a number of TOAs and epoch
c        of period T0.
c     -- written for orbit determination where TOAs are in fact
c        a number of T0s and the resulting epoch is simply an updated
c        version. In principle it can be used for true TOAs as well.
c     -- written to use 'phases.dat' output from circorbit
c     -- uses two methods: my simplex and Andrew stepping idea
c        both methods should procude same results. if not, T0 may
c        not be well determined...
c
c     version 2:  combination of simplex grid search and monte carlo
c                 for each monte carlo loop, we do a simplex
c                 grid-search. That makes it slow, but hopefully good!
c
c
c
c
c     -- direct all questions and bugs to >>>mkramer@jb.man.ac.uk<<<
c
c
c     february 2000 - michael kramer
c

      implicit none

c     -- to handle input...
      integer maxdata, ncolmax
      parameter (maxdata=1000)
      parameter (ncolmax=20)

      real*8 col(ncolmax)

      character filename*30, line*120, numout*80

      integer numcol, i, iargc, narg
      integer tcol, phicol, t0col, dphicol, dt0col
      integer ndata, errors

      real*8 phi(maxdata), dphi(maxdata), t0(maxdata), dt0(maxdata)
      real*8 t(maxdata)
      real*8 pb, dpb, dphiall, pb0, dpb0

c     -- fitting stuff...
      integer nx, ny
      real*8 chi2, chi2best, a, da, b, db
      real*8 abest, dabest, bbest, dbbest, t0best
      real*8 pbbest, dpbbest, t00, t0m, pbstep
      real*8 dpbbestl, dpbbestu, dt0best, dt0bestl, dt0bestu
      real*8 res(maxdata), dres(maxdata), q
      integer nstep, j, n


c     -- the result - now arrays for monte-carlo
      integer ntry, ntrymax, itry, iseed, secnds
      parameter (ntrymax=20000) 
      real*8 pbtry(ntrymax), t0try(ntrymax)
      real*8 phitry(maxdata), gasdev, ran1
      logical distplot, psfile, plot
      
      real*8 twopi
      parameter (twopi=6.283185)


      data filename/'phases.dat'/
      data psfile/.false./
      data distplot/.false./
      data plot/.false./
      data ntry/1000/

      narg=iargc()
      if (narg.eq.0) write(*,*) 'Try -h to see options.'
      do i=1, narg
         call getarg(i, line)

         if (line(1:2).eq.'-h') then
            write(*,111) 
 111        format('USAGE: calct0 [-f<file>] [-h] ',
     +             '[-n<num>] [-dist] [-plot] [-ps]')
            write(*,*) 'Expects file type as produced by circorbit.'
            write(*,*) 'Default file name is hence ''phases.dat''.'
            write(*,*) 
            write(*,*) '-f    - specify file name other than default'
            write(*,*) '-n    - number of MC-loops, default: ',ntry
            write(*,*) '-dist - show MC-distributions'
            write(*,*) '-plot - plot phases vs time'
            write(*,*) '-ps   - write ps-files when plotting'
            write(*,*) '-h    - this message'
            write(*,*) 
            write(*,*) 'After circorbit, just type: calct0'
            write(*,*) 
            write(*,*) 'Questions/bugs -> mkramer@jb.man.ac.uk'
            stop
         endif

         if (line(1:2).eq.'-f') read(line(3:),*) filename
         if (line(1:3).eq.'-ps') psfile=.true.
         if (line(1:5).eq.'-dist') distplot=.true.
         if (line(1:5).eq.'-plot') plot=.true.
         if (line(1:2).eq.'-n') then
            read(line(3:),*) ntry
            if (ntry.gt.ntrymax) then
               write(*,*) 'Maximum number of loops: ',ntrymax
               write(*,*) 'Using this...'
               ntry=ntrymax
            endif
         endif

      enddo

c     structure of input file - we could modify that...
      numcol=5
      tcol=1
      phicol=2
      errors=1
      dphicol=3
      t0col=4
      dt0col=5


c     -- loading --
      open(78,file=filename,form='formatted',status='old',err=900)

 50   continue
      read(78,100) line
 100  format(a120)
      if (line(1:2).eq.'Pb') then    ! read pb from circorbit
         read(line(3:),*) pb0, dpb0
         pb0=pb0/86400.0d0           ! convert to days
         dpb0=dpb0/86400.0d0
         call printnum(pb0,dpb0,numout,i)
         write(*,*) 'previous Pb (d): ',numout(1:i)
         goto 50
      endif
         
      n=0
      rewind(78)
      read(78,100,end=300) line

 200  continue
         read(78,100,end=300) line
         if (line(1:2).eq.'Pb') goto 200

         read(line,*,err=950) (col(i), i=1, numcol)
         n=n+1
         if (n.gt.maxdata) then
            write(*,*) 'Maximum number of data reached: ',maxdata
            stop
         endif
         t(n)=col(tcol)
         phi(n)=col(phicol)

         if (errors.eq.1) then
            dphi(n)=col(dphicol)
         else
            if (errors.eq.2) then
               dphi(n)=dphiall
            else
               dphi(n)=1.0
            endif
         endif

         t0(n)=col(t0col)
         dt0(n)=col(dt0col)

         goto 200
 300  continue
      ndata=n


c     -------------- fitting -------------

c     -- simplex set up --


c     iseed=-643746 ! for those f77-libraries without secnds
      iseed=-int(secnds(0.0))
      dpb=dble(ran1(iseed))  ! just some dummy to initialize

      dpb=0.5*Pb0**2/(t0(ndata)-t0(1))  ! that's the real interval
      t00=t0(1)                         ! some t0 to start with
      

      write(*,*) ntry,' Monte-Carlo-Loops (Simplex) - please wait....'

      do itry=1, ntry           ! monte carlo loop

         if (mod(itry,500).eq.0) write(*,*) 'MC-loop',itry

         do i=1, ndata
c          - use phase-values varied with given uncertainty
            phitry(i)=dphi(i)*dble(gasdev(iseed))+phi(i)
         enddo
         call callsimplex(t,phitry,ndata,pb0,t00,dpb,pbbest,t0best)

         pbtry(itry)=pbbest
         t0try(itry)=t0best

      enddo ! end of MC-loop

      if (distplot) then
         if (psfile) then
            call pgbegin(0,'dist.ps/ps',2,1)
         else
            call pgbegin(0,'/xw',2,1)
         endif
      endif
         
      call checkdist(Pbtry,ntry,pbbest,dpbbest,dpbbestl,dpbbestu,
     +               1.0,distplot)
      call checkdist(t0try,ntry,t0best,dt0best,dt0bestl,dt0bestu,
     +               1.0,distplot)

      if (distplot) call pgend

      write(*,*) 
      write(*,*) 'Simplex result: '
      write(*,*)
      call printnum(pbbest,dpbbest,numout,i)
      write(*,*) 'Pb (d): ',numout(1:i)
      write(*,*) ' lower/upper err: ',dpbbestl,dpbbestu
      call printnum(t0best,dt0best,numout,i)
      write(*,*) 'T0: ',numout(1:i)
      write(*,*) ' lower/upper err: ',dt0bestl, dt0bestu

      write(*,*)


      nx=int(sqrt(float(2*ndata+1)))+1  ! 2*ndata for both methods
      ny=2*ndata/nx+1

      if (plot) then
         if (psfile) then
            call pgbegin(0,'t0.ps/ps',nx,ny)
         else
            call pgbegin(0,'/xw',nx,ny)
         endif
         call pltresult(t,phi,dphi,ndata,pbbest,t0best,1)
      endif

c     ----- convential search -----
      t00=t0(1)
      pbstep=0.5*Pb0**2/(t0(ndata)-t0(1))/200.0
      nstep=max(1,int(dpb0/pbstep))

      write(*,*) 'Convential search: '
      write(*,*)
      write(*,*) 'Period range: ',pb0-dble(nstep)*pbstep,' to ',
     +     pb0+dble(nstep)*pbstep, ' (', nstep,' steps)'

      chi2best=999e99
      do j=-nstep, nstep, 1

         pb=pb0+dble(j)*pbstep ! trial pb
         
         do i=1, ndata

C          I PLOT RES VS T RATHER T0, WHICH I CAN DO too...

cc     -- re-calculate T0 from that...
            t0(i)=t(i)-phi(i)*Pb  
            if (i.eq.1) t00=t0(1)

c     -- model t0 would be...
            n=int((t0(i)-t00)/Pb)
            t0m=t00+float(n)*Pb
         
            res(i)=(t0(i)-t0m)/pb
            if (res(i).gt.0.5) res(i)=res(i)-1.0
            dres(i)=res(I)*sqrt((dpb0/pb)**2+(dt0(i)/t0(i))**2)
c            write(*,*) res(i), dres(i)
         enddo
         call fit(t0,res,ndata,dres,0,a,b,da,db,chi2,q)
         if (chi2.lt.chi2best) then
            chi2best=chi2
            pbbest=pb
            abest=a
            bbest=b
            dabest=a
            dbbest=b
            t0best=t00
         endif
         
      enddo
      q=pbbest
      pbbest=q*(1d0 + bbest*q)
      dpbbest=dbbest*q*q

      call printnum(pbbest,dpbbest,numout,i)
      write(*,*) 'Pb (d): ',numout(1:i)
      call printnum(t0best,dt0best,numout,i)
      write(*,*) 'T0: ',t0best

      if (plot) then
         call pltresult(t,phi,dphi,ndata,pbbest,t0best,2)
         call pgend
      endif

      stop
 900  continue
      write(*,*) 'Could not parse arguments.'
      stop 

 950  continue
      write(*,*) 'Could not read line. Skipping: '
      write(*,*) line
      goto 200

      end
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
      subroutine callsimplex(t, phi, ndata, pb0, t0, dpb, 
     +                       pbbest, t0best)
      
      implicit none

      integer ndata
      real*8 t(ndata), phi(ndata), pb0, t0, dpb, pbbest, t0best

      integer mamax, ma
      parameter (mamax=6)

      real*8 pbstep,chi2best, afit(mamax), dafit(mamax), chi2
      integer nstep, i

      integer maxdata
      parameter (maxdata=1000)
      real*8 dphi(maxdata)

      parameter (nstep=10)
      
      chi2best=999e99
      pbstep=20.0*dpb/dble(nstep)
      ma=2

      do i=-nstep, nstep
c         write(*,*) 'Simplex : ',i+nstep+1
         afit(1)=pb0+dble(i)*pbstep
         afit(2)=t0  
         if (afit(1).ge.0.0) then
            call SimplexFit(t,phi,dphi,ndata,afit,dafit,ma,chi2)
            if (chi2.lt.chi2best) then
               chi2best=chi2
               pbbest=afit(1)
               t0best=afit(2)
            endif
         endif
      enddo
      

      end
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
      subroutine pltresult(x,y,dy,ndata,pb,t0, icall)
      
      implicit none

      integer ndata, i, icall
      real*8 x(ndata), y(ndata), dy(ndata)
      real*8 pb, t0, x1, x2

      do i=1, ndata
         x1=x(i)-0.5*pb
         x2=x(i)+0.5*pb
c         write(*,*) x(i), y(i), dy(i)
         call plotsub(x,y,dy,ndata,pb,t0,x1,x2, icall)
      enddo


      end
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
      subroutine plotsub(x,y,dy,ndata,pb,t0,x1,x2, icall)
      
      implicit none

      integer ndata, i, nplot, icall
      parameter (nplot=500)
      real*8 x(ndata), y(ndata), dy(ndata)
      real*8 pb, t0, x1, x2, y1, y2
      real*8 arg, phi(nplot), xplot(nplot), dx

      dx=(x2-x1)/dble(nplot)
      do i=1, nplot
         xplot(i)=x1+dble(i-1)*dx
         arg=(xplot(i)-t0)/pb
         phi(i)=arg-int(arg)
         if (phi(i).gt.0.5) phi(i)=phi(i)-1.0
         if (phi(i).lt.-0.5) phi(i)=phi(i)+1.0
      enddo

      y1=-0.6
      y2=0.6
      call pgenv8(x1,x2,y1,y2,0,1)
      call pgline8(nplot,xplot,phi)
      call pgsci(1+icall)
      call pgsch(3.0)
      call pgsfs(1)
      call pgpt8(ndata,x,y,3)
      call errorbars(ndata,x,y,dy)
      call pgsci(1)
      call pgsch(1.0)

      end
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c  This is my implementation of the simplex algorithm. At the moment
c  assuming real*8 values.
c
c  Modify the function vertexvalue according to your needs!
c  mk
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
      subroutine SimplexFit(x,y,dy,ndata,a,da,ma,chisq)

      implicit none

c     -- main routine for Simplex-fit:
c        x,y     = data 
c        ndata   = number of data
c        a,da,ma = parameter, their typical error and their number
c

c     type 1
c     -- here an ellipse with axes a,b and offset in a of x0 --
c        a(1)= a
c        a(2)= b
c        a(3)= x0

c     type 2
c     -- just a straight line though with int-function involved
c     a(1)=pb
c     a(2)=t0

      real*8 alpha, beta, gamma, root2
      parameter (alpha=1.0)
      parameter (beta=0.5)
      parameter (gamma=2.0)
      parameter (root2=1.414213562)

      integer maxiter, mamax
      parameter (maxiter=1500)
      parameter (mamax=40)

      integer ndata,ma,i,j,worst,best,icount,endOfIter
      real*8 x,y,dy,a,vertex,center,reflected,expanded
      real*8 contracted,valueOfVertex,vertexValue, chisq
      real*8 dummy,Value,rmin,rmax,step, da, sign, p, q, aorig
      character answer*1

      dimension x(ndata),y(ndata),dy(ndata),a(ma), da(ma)
      dimension vertex(mamax+1,mamax)
      dimension center(mamax),reflected(mamax),expanded(mamax)
      dimension contracted(mamax),valueOfVertex(mamax+1)
      dimension dummy(mamax),p(mamax),q(mamax), aorig(mamax)

      do i=1, ma
         aorig(i)=a(i)
      enddo

c     -- create (ma+1) many vertexes of dimension ma:
c     -- the first vertex contains the starting parameters a:
      do i=1, ma
         vertex(1,i)=a(i)

         if (i.eq.1) step=aorig(1)*0.1
         if (i.eq.2) step=aorig(1)*0.2 ! search in one pb range
         
         p(i)=step*(sqrt(float(ma+1))+ma-1)/(ma*root2)
         q(i)=step*(sqrt(float(ma+1))-1)/(ma*root2)
c         write(*,*) 'Simplex: ',i, step, p(i), q(i)
         a(i)=0.0
      enddo

c     -- leave it for the moment, but we may want to stick to positive a,b,x0
      do i=2, ma+1
         sign=+1.0
         if (mod(i,2).eq.1) sign=-1.0
         do j=1, ma
            vertex(i,j)=vertex(1,j)+q(j)*sign
            
c     --        do not allow negative values
            if (vertex(i,j).le.0) vertex(i,j)=vertex(1,j)

         enddo
         vertex(i,i-1)=vertex(1,i-1)+p(i-1)*sign

c     --        do not allow negative values
         if (vertex(i,i-1).le.0) vertex(i,i-1)=vertex(1,i-1)

      enddo

      
c     -- calculate values of vertexes
      do i=1, ma+1
         ValueOfVertex(i)=VertexValue(x,y,dy,ndata,vertex,
     +                                dummy,ma,i)
      enddo
      
      icount=0
      endOfIter=maxiter

c     -- Begin of iterations loop --
 100  continue

        icount=icount+1
        
c       -- determine worst and best vertex --
        worst=1
        best=1
        do i=2, ma+1
           if (ValueOfVertex(i).lt.ValueOfVertex(best)) best=i
           if (ValueOfVertex(i).gt.ValueOfVertex(worst)) worst=i
        enddo
        
c        if (mod(icount,10).eq.0) then
c           write(*,150) icount,ValueOfVertex(best)
c 150       format('Simplex iteration [',i4,']: (',f20.4,')')
c        endif
        
c     -- check for doneness of iteration
c        if (((ValueOfVertex(best)/ValueOfVertex(worst)).lt.boundary)
c     +       .or.(icount.gt.maxiter)) goto 200
        
        value=2.0*abs(valueofvertex(best)-valueOfVertex(worst))/
     +       (abs(valueofvertex(best))+abs(valueofvertex(worst)))
        if (value.lt.1e-3) goto 200

        if (icount.gt.endOfIter) then
c           answer='y'
c           write(*,160) 
c 160       format(/,5x,'Maximum number of iterations reached!',
c     +            /,5x,'To be continued (y/n) ? ',$)
c           read(*,170) answer
c 170       format(a1)
c           if (answer.eq.'n') goto 200
c           endOfIter=endOfIter+maxiter
           goto 200
        endif


c       -- recflect worst vertex on surface including all but worst
c          vertex:

c       1. calculate center:
        do i=1, ma
           center(i)=0.0
        enddo
        do i=1, ma+1
           if (i.ne.worst) then
              do j=1, ma
                 center(j)=center(j)+vertex(i,j)
              enddo
           endif
        enddo
        do i=1, ma
           center(i)=center(i)/ma
        enddo

c     2. calculate reflected vertex:
        do i=1, ma
           reflected(i)=(1.0+alpha)*center(i)-alpha*vertex(worst,i)
        enddo

c     -- compare reflected vertex to best one

        Value=vertexValue(x,y,dy,ndata,vertex,reflected,ma,-1)

        if (Value.le.valueOfVertex(best)) then
        
           call replace(vertex,reflected,ma,worst)
           ValueOfVertex(worst)=Value

c          - it's better => try expansion ! -
           do i=1, ma
              expanded(i)=(1.0+gamma)*center(i)-gamma*vertex(worst,i)
           enddo
           Value=vertexValue(x,y,dy,ndata,vertex,expanded,ma,-1)

c          - if expansion still better => replace worst by it
           if (Value.le.ValueOfVertex(best)) then
              call replace(vertex,expanded,ma,worst)
              ValueOfVertex(worst)=Value
           endif

c     -- reflected was worse than best: compare it now to worst --
        else
           if (Value.le.ValueOfVertex(worst)) then
c              - it is better than worst, just accept it:
              call replace(vertex,reflected,ma,worst)
              ValueOfVertex(worst)=Value
           else
c              - it's a pity, it is worse than worst => try contraction
              do i=1, ma
                 contracted(i)=(1.0-beta)*center(i)+beta*vertex(worst,i)
              enddo
              Value=vertexValue(x,y,dy,ndata,vertex,
     +                          contracted,ma,-1)

              if (Value.le.ValueOfVertex(worst)) then

c                 - if contraction does it, replace worst:
                 call replace(vertex,contracted,ma,worst)
                 valueOfVertex(worst)=Value

              else

c               - deep trouble: only way out: shrink all vertexes but best
                 do i=1, ma+1
                    if (i.ne.best) then
                       do j=1, ma
                          vertex(i,j)= 0.5*(vertex(i,j)+vertex(best,j))
                       enddo
                       ValueOfVertex(i)=vertexValue(x,y,dy,ndata,
     +                                       vertex,dummy,ma,i)
                    endif
                 enddo
                 
              endif
           endif
        endif            

        goto 100

 200  continue
      

c     - calculate largest and smallest value of one parameter and the
c       relative deviation (temporarily stored in dummy)
      do j=1, ma
         rmin=vertex(1,j)
         rmax=vertex(1,j)
         do i=2, ma+1
            if (vertex(i,j).lt.rmin) rmin=vertex(i,j)
            if (vertex(i,j).gt.rmax) rmax=vertex(i,j)
         enddo
         dummy(j)=2.0*(rmax-rmin)/(rmax+rmin)
      enddo
      
c     -- calculate mean of vertexes as result of fit --
      do j=1, ma
         do i=1, ma+1
            a(j)=a(j)+vertex(i,j)
         enddo
      enddo
      do i=1, ma
         a(i)=a(i)/(ma+1)
c        -- da contains squares of absolute errors: --
         da(i)=(a(i)*dummy(i))**2
      enddo

      chisq=vertexValue(x,y,dy,ndata,vertex,a,ma,-1)

      end
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccca
      subroutine replace(vertexarray,vertex,ma,No)

      implicit none
      
c     -- this subroutine replaces Vertex #No of vertexArray by vertex
      
      integer mamax
      parameter (mamax=40)

      real*8 vertexarray,vertex
      integer ma,No,i

      dimension vertexArray(mamax+1,mamax),vertex(ma)

      do i=1, ma
         vertexArray(No,i)=vertex(i)
      enddo
      
      end
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccacccccccc
      real*8 function vertexValue(x,y,dy,ndata,vertex,dummy,ma,No)

c     -- this subroutine calculates the vcorrespondending value of 
c        vertex #No, if No>0, or of vertex dummy, if No<0

      implicit none

      integer mamax
      parameter (mamax=40)

      real*8 x,y,dy,vertex,fitvalue,chi,dummy,a
      integer ndata,ma, no, i

      dimension x(ndata),y(ndata),dy(ndata),vertex(mamax+1,mamax),
     +          a(mamax),dummy(ma)

      real*8 pb, t0, arg, phi ! type 2
      
      do i=1, ma
         if (No.gt.0) then
            a(i)=vertex(No,i)
         else
            a(i)=dummy(i)
         endif
      enddo

      chi=0.0

      pb=a(1)
      t0=a(2)
      do i=1, ndata
         arg=(x(i)-t0)/pb
         phi=arg-int(arg)
         if (phi.gt.0.5) phi=phi-1.0
         if (phi.lt.-0.5) phi=phi+1.0
c        no error estimate for y since we are doing MC
c        fitvalue=(y(i)-phi)/dy(i)
         fitvalue=(y(i)-phi)
         
         if (pb.le.0.0) fitvalue=9999e9
         if (t0.le.0.0) fitvalue=9999e9
         chi=chi+fitvalue**2
      enddo
      
      vertexValue=chi
      
      end
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc













